/*
** glscissor.c
** 
** Written by Ian Buck
** 6/15/2000
** http://graphics.stanford.edu/software/wiregl
**
** Copyright 2000 
** The Board of Trustees of The Leland Stanford Junior University.
** All rights reserved.
**
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or
** modify this software, provided that the above copyright notice and
** this permission notice appear in all copies of this software.  No
** part of this software or any derivatives thereof may be used in
** graphics systems for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind,
** express, implied or otherwise, including without limitation, any
** warranty of merchantability or fitness for a particular purpose.
*/

#include "glcontext.h"
#include "glerror.h"
#include "glhw.h"

void 
__glscissor_initbits (GLscissorbits *s, GLconfig *cfg) {
	/* do nothing */
	UNUSED(s);
	UNUSED(cfg);
}

void
__glscissor_clamp (GLscissorstate *current, GLscissorstate *target, GLrecti *p) {
	p->x1 = target->x;
	p->y1 = target->y;
	p->x2 = target->x + target->width;
	p->y2 = target->y + target->height;

	/* Clamp it to the outputdims */
	if (p->x1 < current->outputdims.x1) p->x1 = current->outputdims.x1;
	if (p->y1 < current->outputdims.y1) p->y1 = current->outputdims.y1;
	if (p->x1 > current->outputdims.x2) p->x1 = current->outputdims.x2;
	if (p->y1 > current->outputdims.y2) p->y1 = current->outputdims.y2;

	if (p->x2 < current->outputdims.x1) p->x2 = current->outputdims.x1;
	if (p->y2 < current->outputdims.y1) p->y2 = current->outputdims.y1;
	if (p->x2 > current->outputdims.x2) p->x2 = current->outputdims.x2;
	if (p->y2 > current->outputdims.y2) p->y2 = current->outputdims.y2;
	
	if (p->x2 == p->x1 ||
		p->y2 == p->y1) {
		p->x1 = 0;
		p->x2 = 0;
		p->y1 = 0;
		p->y2 = 0;
	} else {
		p->x1 -= current->outputdims.x1;
		p->y1 -= current->outputdims.y1;
		p->x2 -= current->outputdims.x1;
		p->y2 -= current->outputdims.y1;
	}
}

void
__glscissor_init (GLscissorstate *s, GLconfig *c) {
	int id;
	s->scissortest = GL_FALSE;
	s->x = 0;
	s->y = 0;
	s->width = GL_MAXINT;
	s->height = GL_MAXINT;


	id = c->id;
	if (id >= 0 && c->numprojectors > 0)
		s->outputdims = c->bounds[id];
	else
		s->outputdims = c->viewport;

	if (!c->localviewport.x2) 
		s->widthscale = 1.0f;
	else
		s->widthscale = ((GLfloat) c->viewport.x2 - c->viewport.x1) / ((GLfloat)c->localviewport.x2);

	if (!c->localviewport.y2)
		s->heightscale = 1.0f;
	else
		s->heightscale = ((GLfloat) c->viewport.y2 - c->viewport.y1) / ((GLfloat)c->localviewport.y2);

}

void
__glscissor_destroy (GLscissorstate *s) {
	/* do nothing */
	UNUSED(s);
}

void GLSTATE_DECL
__glstate_Scissor (GLint x, GLint y, 
					 GLsizei width, GLsizei height) {
	GLcontext *g = GetCurrentContext();
	GLscissorstate *s = &(g->scissor);
	GLstatebits *stateb = GetStateBits();
	GLscissorbits *sb = &(stateb->scissor);

	if (g->current.beginend) 
		if (__glerror(__LINE__, __FILE__, GL_INVALID_OPERATION,
			"glScissor called in begin/end"))
			return;

	if (IsBucketingActive()) 
		g->bucket.flush(g);

	if (width < 0 || height < 0)
		if (__glerror(__LINE__, __FILE__, GL_INVALID_VALUE,
			"glScissor called with negative width/height: %d,%d",
			width, height))
			return;

	s->x = (GLint) (x*s->widthscale);
	s->y = (GLint) (y*s->heightscale);
	s->width = (GLint) (width*s->widthscale);
	s->height = (GLint) (height*s->heightscale);

	sb->rect = g->nbitID;
	sb->dirty = g->nbitID;
}

void GLSTATE_DECL
__gltrack_Scissor (GLint x, GLint y, 
					 GLsizei width, GLsizei height) {
	GLcontext *g = GetCurrentContext();
	GLscissorstate *s = &(g->scissor);
	GLstatebits *stateb = GetStateBits();
	GLscissorbits *sb = &(stateb->scissor);

	__glhw_Scissor (x, y, width, height);

	s->x = x;
	s->y = y;
	s->width = width;
	s->height = height;

	sb->rect = g->nbitID;
	sb->dirty = g->nbitID;
}

