/* 
** pipe_server/disable.c 
** http://graphics.stanford.edu/software/wiregl
**
** Copyright 2000
** The Board of Trustees of The Leland Stanford Junior University.
** All rights reserved.
**
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or
** modify this software, provided that the above copyright notice and
** this permission notice appear in all copies of this software.  No
** part of this software or any derivatives thereof may be used in
** graphics systems for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind,
** express, implied or otherwise, including without limitation, any
** warranty of merchantability or fitness for a particular purpose.
*/

#ifdef WINDOWS
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif
#include <GL/gl.h>

#include "wiregl/include/wireGL.h"
#include "wiregl/include/wiregl_pipe.h"
#include "wiregl/glcontext/glcontext.h"

void __decodeDisable( void )
{
	GLenum cap =  READ_DATA( 0, GLenum );

	// If we're on L2 and the user turns off the scissor test,
	// we'd better not really turn it off because otherwise
	// glClear() will blow away the strip headers.  Instead, just
	// set it to enclose the whole tile.
	
	if ( wiregl_pipe.use_l2 && cap == GL_SCISSOR_TEST )
	{
		glScissor( 2, wiregl_pipe.actual_window_height - wiregl_pipe.window_height, 
							 wiregl_pipe.window_width, wiregl_pipe.window_height );
	}

	__gltrack_Disable( cap );
	INCR_DATA_PTR( 4 );
}

void __decodeEnable( void )
{
	GLenum cap =  READ_DATA( 0, GLenum );

	// If we're on L2 and the scissor test is being enabled, we'd
	// better re-set the scissor rectangle because disabling it has
	// a strange side effect (see glDisable above).

	if (wiregl_pipe.use_l2 && cap == GL_SCISSOR_TEST )
	{
		GLcontext *c = __glGetCurrentContext();
		if (c->scissor.width == GL_MAXINT)
		{
			// the user has never set the scissor rectangle.  This means
			// that the current scissor rectangle on the server is equal
			// to the L2 tile, which means we can get away with doing
			// exactly nothing here.
		}
		else
		{
			// We already offset the scissor rectangle before we called the
			// state tracker (see viewport.c), so we can just use the 
			// tracked values directly.  Whoopee!

			glScissor( c->scissor.x, c->scissor.y, c->scissor.width, c->scissor.height );
		}
	}

	__gltrack_Enable( cap );
	INCR_DATA_PTR( 4 );
}
