/* 
** pipe_server/server_net.c 
** http://graphics.stanford.edu/software/wiregl
**
** Copyright 2000
** The Board of Trustees of The Leland Stanford Junior University.
** All rights reserved.
**
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or
** modify this software, provided that the above copyright notice and
** this permission notice appear in all copies of this software.  No
** part of this software or any derivatives thereof may be used in
** graphics systems for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind,
** express, implied or otherwise, including without limitation, any
** warranty of merchantability or fitness for a particular purpose.
*/

#ifdef WINDOWS
#define WIN32_LEAN_AND_MEAN
#include <winsock.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#endif

#include <stdio.h>
#include <errno.h>

#include "wiregl/include/wiregl_pipe.h"
#include "wiregl/include/wiregl_protocol.h"
#include "wiregl/include/wiregl_util.h"
#include "wiregl/include/wiregl_common.h"

static void connect_thread( void *arg )
{
	WireGLServer *server = wireGLBecomeServer( wiregl_pipe.port );
	WireGLConnectionRequest *request = NULL;

	(void) arg;

	for ( ; ; )
	{
		WireGLConnection *conn = NULL;
		WireGLConnection *peer_conn = NULL;

		if (wiregl_pipe.fname != NULL)
		{
#if 1
			wireGLAbort( );
#else
			conn = wireGLAcceptFileClient( wiregl_pipe.fname );
#endif
		}
		else
		{
			conn = wireGLAcceptClient( server, &request );

#if 0
			if ( request.pipe_buffer_size > ? )
			{
			}
#endif

			wiregl_pipe.window_x = request->window_x;
			wiregl_pipe.window_y = request->window_y;
			wiregl_pipe.window_width = request->window_width;
			wiregl_pipe.window_height = request->window_height;
			wiregl_pipe.use_l2 = request->use_l2;
			wiregl_pipe.depth_bits = request->depth_bits;
			wiregl_pipe.stencil_bits = request->stencil_bits;
			wiregl_pipe.feather = request->feather;
		}

		peer_conn = NULL;
		if (*request->peer) {
			peer_conn = wireGLConnectToServer( request->peer, WIREGL_TCPIP_DEFAULT_PORT,
											   request+1 );
			wireGLFinishConnectToServer( peer_conn );
		}

		wireGLFinishAcceptClient( conn, request );
		wireGLPipeserverConnect( conn, peer_conn );
		wireGLReleaseSemaphore( &wiregl_pipe.someone_connected );

		if ( wiregl_pipe.fname != NULL )
		{
			// Better do only one file client.
			break;
		}
	}
}

void wireGLStartConnectionThread( )
{
	wireGLInitSemaphore( &wiregl_pipe.someone_connected, 0 );
	wireGLCreateThread( connect_thread, NULL );
	wireGLAcquireSemaphore( &wiregl_pipe.someone_connected );
}


