/* 
** pipe_server/viewport.c 
** http://graphics.stanford.edu/software/wiregl
**
** Copyright 2000
** The Board of Trustees of The Leland Stanford Junior University.
** All rights reserved.
**
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or
** modify this software, provided that the above copyright notice and
** this permission notice appear in all copies of this software.  No
** part of this software or any derivatives thereof may be used in
** graphics systems for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind,
** express, implied or otherwise, including without limitation, any
** warranty of merchantability or fitness for a particular purpose.
*/

#ifdef WINDOWS
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif
#include <GL/gl.h>

#include "wiregl/include/wiregl_protocol.h"
#include "wiregl/include/wiregl_pipe.h"

#if 0
static int __getUnNormalizedCoords( GLint *x, GLint *y, GLsizei *width, GLsizei *height )
{
	int has_useful_data = READ_DATA( 0, int );
	if (has_useful_data)
	{
		float nx = READ_DATA( 4, float );
		float ny = READ_DATA( 8, float );
		float nwidth = READ_DATA( 12, float );
		float nheight = READ_DATA( 16, float );
		unsigned int pipe_width, pipe_height;
#ifdef WINDOWS
		RECT r;

		GetClientRect( wiregl_pipe.hWnd, &r );
		pipe_width = r.right-r.left;
		pipe_height = r.bottom-r.top;
#else
		Window root;
		int silly_x, silly_y;
		unsigned int border, depth;

		XGetGeometry( wiregl_pipe.dpy, wiregl_pipe.win, 
				&root, &silly_x, &silly_y,
				&pipe_width, &pipe_height, &border, &depth );
#endif
		*x = (int) (nx*wiregl_pipe.mural_width);
		*y = (int) (ny*wiregl_pipe.mural_height);
		*width = (int) (nwidth*wiregl_pipe.mural_width);
		*height = (int) (nheight*wiregl_pipe.mural_height);

#if 1
		/* Changed by IB */
		{
			int x1, y1, x2, y2;
			x1 = *x;
			x2 = *x + *width;
			y1 = *y;
			y2 = *y + *height;

			if (x1 < wiregl_pipe.mural_x)
				x1 = 0;
			else if (x1 > wiregl_pipe.mural_x + wiregl_pipe.window_width)
				x1 = wiregl_pipe.mural_x + wiregl_pipe.window_width;
			else {
				x1 -= wiregl_pipe.mural_x;
				x1 *= (int) (x1 * ((float)pipe_width)/((float)wiregl_pipe.window_width));
			}
			if (y1 < wiregl_pipe.mural_y)
				y1 = 0;
			else if (y1 > wiregl_pipe.mural_y + wiregl_pipe.window_height)
				y1 = wiregl_pipe.mural_y + wiregl_pipe.window_height;
			else {
				y1 -= wiregl_pipe.mural_y;
				y1 *= (int) (y1 * ((float)pipe_height)/((float)wiregl_pipe.window_height));
			}
			if (x2 < wiregl_pipe.mural_x)
				x2 = 0;
			else if (x2 > wiregl_pipe.mural_x + wiregl_pipe.window_width)
				x2 = wiregl_pipe.mural_x + wiregl_pipe.window_width;
			else {
				x2 -= wiregl_pipe.mural_x;
				x2 = (int) (x2 * ((float)pipe_width)/((float)wiregl_pipe.window_width));
			}
			if (y2 < wiregl_pipe.mural_y)
				y2 = 0;
			else if (y2 > wiregl_pipe.mural_y + wiregl_pipe.window_height)
				y2 = wiregl_pipe.mural_y + wiregl_pipe.window_height;
			else {
				y2 -= wiregl_pipe.mural_y;
				y2 *= (int) (y2 * ((float)pipe_height)/((float)wiregl_pipe.window_height));
			}

			*x = x1;
			*y = y1;
			*width = x2-x1;
			*height = y2-y1;
		}
#else
		/* Old viewport code */
		*x -= wiregl_pipe.mural_x;
		*y -= wiregl_pipe.mural_y;

		*width = (int) (*width * ((float)pipe_width)/((float)wiregl_pipe.window_width));
		*height = (int) (*height * ((float)pipe_height)/((float)wiregl_pipe.window_height));
#endif
	}
	INCR_DATA_PTR( 20 );
	return has_useful_data;
}
#endif

void __decodeViewport( void )
{
	GLint x = READ_DATA( 0, GLint );
	GLint y = READ_DATA( 4, GLint );
	GLsizei width = READ_DATA( 8, GLint );
	GLsizei height = READ_DATA( 12, GLint );

	if ( wiregl_pipe.use_l2 )
	{
		x += 2; // skip the strip headers
		y += wiregl_pipe.actual_window_height - wiregl_pipe.window_height;
	}

	__gltrack_Viewport( x, y, width, height );
	INCR_DATA_PTR( 16 );
}

void __decodeScissor( void )
{
	GLint x = READ_DATA( 0, GLint );
	GLint y = READ_DATA( 4, GLint );
	GLsizei width = READ_DATA( 8, GLint );
	GLsizei height = READ_DATA( 12, GLint );

	if ( wiregl_pipe.use_l2 )
	{
		x += 2; // skip the strip headers
		y += wiregl_pipe.actual_window_height - wiregl_pipe.window_height;
	}

	__gltrack_Scissor( x, y, width, height );
	INCR_DATA_PTR( 16 );
}
