/*
** util/gm.c
** Generated by opengl_stub/windows_exports.py
** http://graphics.stanford.edu/software/wiregl
**
** Copyright 2000 
** The Board of Trustees of The Leland Stanford Junior University.
** All rights reserved.
**
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or
** modify this software, provided that the above copyright notice and
** this permission notice appear in all copies of this software.  No
** part of this software or any derivatives thereof may be used in
** graphics systems for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind,
** express, implied or otherwise, including without limitation, any
** warranty of merchantability or fitness for a particular purpose.
**
*/

#include "__utilsource.h"

#include <stdio.h>
#include <stdlib.h>
#include <gm.h>
#ifndef WINDOWS
#include <ctype.h> // for tolower
#endif

#include "wiregl/include/wiregl_util.h"
#include "wiregl/include/wiregl_common.h"
#include "wiregl/include/wiregl_protocol.h"

typedef struct WireGLMultiBuffer {
	unsigned int  len;
	unsigned int  max;
	void         *buf;
} WireGLMultiBuffer;

static struct {
	unsigned int          initialized;
	struct gm_port       *port;
	unsigned int          node_id;
	unsigned int          num_nodes;
	WireGLMultiBuffer    *multi;
	unsigned char        *enable;
	unsigned int          message_size;
	WireGLNetReceiveFunc  recv;
	WireGLNetCloseFunc    close;
	WireGLBufferPool      read_pool;
	WireGLBufferPool      write_pool;
	WireGLBufferPool      unpinned_read_pool;
	WireGLBufferPool      flow_control_pool;
	unsigned int          num_outstanding_writes;
	unsigned int          num_outstanding_flow;
	unsigned int          num_send_tokens;
} wiregl_gm;

#define WIREGL_GM_PINNED_READ_MEMORY_SIZE   ( 8 << 20 )
#define WIREGL_GM_PINNED_WRITE_MEMORY_SIZE  ( 8 << 20 )

#define WIREGL_GM_PORT               4
#define WIREGL_GM_API_VERSION        GM_API_VERSION_1_0
#define WIREGL_GM_PRIORITY           GM_LOW_PRIORITY
#define WIREGL_GM_DEBUG              0

#define WIREGL_GM_FLOW_CONTROL_LENGTH  8
#define WIREGL_GM_FLOW_CONTROL_SIZE    4
/* magic number is 'GmFC' */
#define WIREGL_GM_FLOW_CONTROL_MAGIC   0x476d4643

static void 
wiregl_gm_info( void )
{
    char name[ GM_MAX_HOST_NAME_LEN ];
    unsigned int node_id, max_node_id, min_size;

    gm_get_host_name( wiregl_gm.port, name );
    gm_get_node_id( wiregl_gm.port, &node_id );
    wireGLWarning( WIREGL_WARN_DEBUG, "GM: name=\"%s\" id=%u",
                   name, node_id );

    gm_max_node_id_inuse( wiregl_gm.port, &max_node_id );
    wireGLWarning( WIREGL_WARN_DEBUG, "GM: gm_max_node_id_inuse=%u",
                   max_node_id );

    min_size = gm_min_size_for_length( __wiregl_max_send );
    wireGLWarning( WIREGL_WARN_DEBUG, "GM: gm_min_size_for_length( "
                   "__wiregl_max_send=%u ) = %u", __wiregl_max_send,
                   min_size );

    wireGLWarning( WIREGL_WARN_DEBUG, "GM: gm_max_length_for_size( "
                   "size=%u ) = %u", min_size,
                   gm_max_length_for_size( min_size ) );

	wireGLWarning( WIREGL_WARN_DEBUG, "GM: gm_min_message_size( ) = %u",
				   gm_min_message_size( wiregl_gm.port ) );

    wireGLWarning( WIREGL_WARN_DEBUG, "GM: num_send_tokens=%u "
                   "num_receive_tokens=%u",
				   gm_num_send_tokens( wiregl_gm.port ),
                   gm_num_receive_tokens( wiregl_gm.port ) );
}

#if WIREGL_GM_DEBUG

static void 
wiregl_gm_check_recv_event( gm_recv_event_t *event )
{
	switch ( event->recv.type ) {
	  case GM_NO_RECV_EVENT:
	  case GM_ALARM_EVENT:
	  case GM_SENT_EVENT:
	  case _GM_SLEEP_EVENT:
	  case GM_RAW_RECV_EVENT:
	  case GM_RECV_EVENT:
	  case GM_HIGH_RECV_EVENT:
	  case GM_PEER_RECV_EVENT:
	  case GM_HIGH_PEER_RECV_EVENT:
	  case _GM_DIRECTED_SEND_NOTIFICATION_EVENT:
	  case _GM_FLUSHED_ALARM_EVENT:
	  case GM_SENT_TOKENS_EVENT:
	  case GM_IGNORE_RECV_EVENT:
		/* ignore */
		break;

	  case GM_FAST_RECV_EVENT:
	  case GM_FAST_HIGH_RECV_EVENT:
	  case GM_FAST_PEER_RECV_EVENT:
	  case GM_FAST_HIGH_PEER_RECV_EVENT:
		wireGLWarning( WIREGL_WARN_DEBUG, "GM fast event (type=%u)",
					   event->recv.type );
		break;

	  case GM_SENDS_FAILED_EVENT:
	  case GM_BAD_SEND_DETECTED_EVENT:
	  case GM_SEND_TOKEN_VIOLATION_EVENT:
	  case GM_RECV_TOKEN_VIOLATION_EVENT:
	  case GM_BAD_RECV_TOKEN_EVENT:
	  case GM_ALARM_VIOLATION_EVENT:
	  case GM_REJECTED_SEND_EVENT:
	  case GM_ORPHANED_SEND_EVENT:
	  case GM_BAD_RESEND_DETECTED_EVENT:
	  case GM_DROPPED_SEND_EVENT:
	  case GM_BAD_SEND_VMA_EVENT:
	  case GM_BAD_RECV_VMA_EVENT:
		wireGLWarning( WIREGL_WARN_NOTICE, "GM error event (type=%u)",
					   event->recv.type );
		break;

	  default:
		wireGLWarning( WIREGL_WARN_CRITICAL, "GM unknown event (type=%u)",
					   event->recv.type );
		break;
	}
}

#endif

static const char *
wiregl_gm_str_error( gm_status_t status )
{
    const char *description;

#define CASE(error, desc) case error : description = desc; break

	switch ( status )
	{
		CASE (GM_SUCCESS, "success");
		CASE (GM_FAILURE, "unqualified error");
		CASE (GM_INPUT_BUFFER_TOO_SMALL, "input buffer is too small");
		CASE (GM_OUTPUT_BUFFER_TOO_SMALL, "output buffer is too small");
		CASE (GM_TRY_AGAIN, "try again");
		CASE (GM_BUSY, "busy");
		CASE (GM_MEMORY_FAULT, "memory fault");
		CASE (GM_INTERRUPTED, "interrupted");
		CASE (GM_INVALID_PARAMETER, "invalid parameter");
		CASE (GM_OUT_OF_MEMORY, "out of memory");
		CASE (GM_INVALID_COMMAND, "invalid command");
		CASE (GM_PERMISSION_DENIED, "permission denied");
		CASE (GM_INTERNAL_ERROR, "internal error");
		CASE (GM_UNATTACHED, "unattached");
		CASE (GM_UNSUPPORTED_DEVICE, "unsupported device");
		CASE (GM_SEND_TIMED_OUT, "send timed out");
		CASE (GM_SEND_REJECTED, "send was rejected by receiver");
		CASE (GM_SEND_TARGET_PORT_CLOSED, "target port was closed");
		CASE (GM_SEND_TARGET_NODE_UNREACHABLE, "target node was unreachable");
		CASE (GM_SEND_DROPPED, "send dropped as requested by the client");
		CASE (GM_SEND_PORT_CLOSED, "send port was closed");
		CASE (GM_NODE_ID_NOT_YET_SET,
			  "node ID not set (node has not been configured by the mapper)");
		CASE (GM_STILL_SHUTTING_DOWN,
			  "port is still shutting down from previous open");
		CASE (GM_CLONE_BUSY, "clone device is busy");
	  default:
		description = "unknown error";
	}

	return description;
}

static void *
wiregl_gm_dma_malloc( unsigned long length )
{
	void *buf;
	gm_status_t status;

	buf = gm_dma_malloc( wiregl_gm.port, length );
	if ( buf == NULL )
	{
		wireGLError( "gm_dma_malloc( port=0x%p, len=%d ) : failed",
					 wiregl_gm.port, length );
	}

#if WIREGL_GM_DEBUG
	wireGLWarning( WIREGL_WARN_DEBUG, "GM: dma_malloc, length=%d "
				   "range=0x%p:0x%p", length, buf,
				   (unsigned char*) buf + length );
#endif

#if 1
	/* Not entirely sure why we are doing this, but we've had a lot of
       trouble with the gm_dma_malloc() returning a non-NULL pointer,
       but not having actually locked the memory.  Whoops! */
	status = gm_register_memory( wiregl_gm.port, buf, length );
	if ( status != GM_SUCCESS )
	{
		wireGLError( "GM: gm_register_memory( port, buf=0x%p, len=%d ) : "
					 "error=%s (%d)", buf, length,
					 wiregl_gm_str_error( status ), status );
	}
#endif

	return buf;
}

UTIL_DECL void *
wireGLGmAlloc( void )
{
	void *buf = wireGLBufferPoolPop( &wiregl_gm.write_pool );
	while ( buf == NULL )
	{
		if ( wiregl_gm.num_outstanding_writes == 0 )
		{
			wireGLSimpleError( "GM: wireGLGmAlloc - no available buffers, "
							   "and none outstanding." );
		}
		wireGLGmRecv( );
		buf = wireGLBufferPoolPop( &wiregl_gm.write_pool );
	}
	return buf;
}

static void 
wiregl_gm_free( void *buf )
{
	wireGLBufferPoolPush( &wiregl_gm.read_pool, buf );
}

static void 
wiregl_gm_free_unpinned( void *buf )
{
	wireGLBufferPoolPush( &wiregl_gm.unpinned_read_pool, buf );
}

static void
wireGLGmRecvFlowControl( gm_recv_event_t *event )
{
	unsigned short node_id = ntohs( event->recv.sender_node_id );
	unsigned long  length  = ntohl( event->recv.length );
	unsigned int  *data    = (unsigned int *) gm_ntohp( event->recv.buffer );
#if WIREGL_GM_DEBUG
	wireGLWarning( WIREGL_WARN_DEBUG, "GM: received flow control ",
				   "node=%d, buf=0x%p, len=%d", node_id,
				   data, length );
#endif
	wireGLAssert( node_id < wiregl_gm.num_nodes && 
				  length == WIREGL_GM_FLOW_CONTROL_LENGTH &&
				  data[0] == WIREGL_GM_FLOW_CONTROL_MAGIC );
	((void) length);

	wiregl_gm.enable[node_id] = (unsigned char) ( data[1] != 0 );
	data[0] = 0xdeadbeef;
	gm_provide_receive_buffer( wiregl_gm.port, data,
							   WIREGL_GM_FLOW_CONTROL_SIZE,
							   WIREGL_GM_PRIORITY );
}

static void
wireGLGmRecvMulti( WireGLMessageMulti *msg, unsigned int len, 
				   WireGLMultiBuffer *multi )
{
	WireGLMessageType type = msg->type;
	unsigned char *src, *dst;

	if ( len + multi->len > multi->max )
	{
		if ( multi->max == 0 )
		{
			multi->len = 0;
			multi->max = 8192;
		}
		while ( len + multi->len > multi->max )
		{
			multi->max <<= 1;
		}
		wireGLRealloc( &multi->buf, multi->max );
	}

	dst = (unsigned char *) multi->buf + multi->len;
	src = (unsigned char *) msg + sizeof(*msg);
	len -= sizeof(*msg);
	memcpy( dst, src, len );
	multi->len += len;

	gm_provide_receive_buffer( wiregl_gm.port, msg,
							   wiregl_gm.message_size,
							   WIREGL_GM_PRIORITY );

	if ( type == WIREGL_MESSAGE_MULTI_TAIL )
	{
		void *msg = multi->buf;
		int   len = multi->len;
				  
		multi->buf = NULL;
		multi->len = 0;
		multi->max = 0;

		wiregl_gm.recv( msg, len, wireGLFree );
	}
}

static void
wireGLGmRecvOther( WireGLMessage *msg, unsigned int len )
{
	void *temp = wireGLBufferPoolPop( &wiregl_gm.read_pool );

	if ( temp == NULL )
	{
#if WIREGL_GM_DEBUG
		wireGLWarning( WIREGL_WARN_DEBUG, "wireGLGmRecv: ran out of "
					   "pinned memory, copying to unpinned" );
#endif

		/* we're out of pinned buffers, copy this message into
		 * an unpinned buffer so we can jam the just received
		 * buffer back in the hardware */
		temp = wireGLBufferPoolPop( &wiregl_gm.unpinned_read_pool );
		if ( temp == NULL )
			temp = wireGLAlloc( __wiregl_max_send );
		memcpy( temp, msg, len );

		*(unsigned int *) msg = 0xdeadbeef;
		gm_provide_receive_buffer( wiregl_gm.port, msg, 
								   wiregl_gm.message_size,
								   WIREGL_GM_PRIORITY );

		wiregl_gm.recv( temp, len, wiregl_gm_free_unpinned );
	}
	else
	{
		*(unsigned int *) temp = 0xdeadbeef;
		gm_provide_receive_buffer( wiregl_gm.port, temp, 
								   wiregl_gm.message_size,
								   WIREGL_GM_PRIORITY );

		wiregl_gm.recv( msg, len, wiregl_gm_free );
	}
}

UTIL_DECL int
wireGLGmRecv( void )
{
	gm_recv_event_t *event = gm_receive( wiregl_gm.port );

	if ( event->recv.type == GM_NO_RECV_EVENT )
		return 0;

#if WIREGL_GM_DEBUG
	wiregl_gm_check_recv_event( event );
#endif

	switch ( event->recv.type ) {

	  case GM_RECV_EVENT:
	  case GM_HIGH_RECV_EVENT:
	  case GM_PEER_RECV_EVENT:
	  case GM_HIGH_PEER_RECV_EVENT:
		if ( event->recv.size == WIREGL_GM_FLOW_CONTROL_SIZE )
		{
			wireGLGmRecvFlowControl( event );
		}
		else
		{
		  unsigned int   len = ntohl( event->recv.length );
		  WireGLMessage *msg = (WireGLMessage *) gm_ntohp( event->recv.buffer );
		  unsigned short src = ntohs( event->recv.sender_node_id );
		  WireGLMultiBuffer *multi;

		  wireGLAssert( src < wiregl_gm.num_nodes );
		  multi = &wiregl_gm.multi[src];

#if WIREGL_GM_DEBUG
		  wireGLWarning( WIREGL_WARN_DEBUG, "GM: received message, node=%d,"
						 "buf=0x%p, len=%d", node_id, buf, length );
#endif

		  switch ( msg->type )
		  {
			case WIREGL_MESSAGE_MULTI_BODY:
			case WIREGL_MESSAGE_MULTI_TAIL:
			  wireGLGmRecvMulti( &msg->multi, len, multi );
			  break;
			  
			default:
			  wireGLAssert( multi->buf == NULL );
			  wireGLGmRecvOther( msg, len );
		  }
		}
		break;

	  case GM_FAST_RECV_EVENT:
	  case GM_FAST_HIGH_RECV_EVENT:
		/* can't handle these yet, I don't know if we need to call
           gm_provide_receive_buffer() after handling these or not */

	  default:
		gm_unknown( wiregl_gm.port, event );
	}

	return 1;
}

static void 
wiregl_gm_send_callback( struct gm_port *port, void *ctx, gm_status_t status )
{
	void *buf = ctx;

    if ( status != GM_SUCCESS )
    {
        wireGLSimpleError( "wiregl_gm_send_callback error: %s (%d)",
                           wiregl_gm_str_error( status ), status );
    }

#if WIREGL_GM_DEBUG
	wireGLWarning( WIREGL_WARN_DEBUG, "GM: send callback, buf=0x%p", buf );
#endif

	wireGLBufferPoolPush( &wiregl_gm.write_pool, buf );

	wireGLAssert( wiregl_gm.num_outstanding_writes > 0 );
	wiregl_gm.num_outstanding_writes--;
	wiregl_gm.num_send_tokens++;
	((void) port);
}

UTIL_DECL void 
wireGLGmSend( WireGLConnection *conn, void **bufp, 
			  void *start, unsigned int len )
{
	/* Every time the client calls send we try to be polite and grab
       and pending GM messages.  We also have to block if our
       flow-control is turned off or we don't have any send tokens.
       If something is breaking in GM I'd certainly look here
       first. */
	while ( wireGLGmRecv( ) ||
			!wiregl_gm.enable[ conn->gm_node_id ] ||
			wiregl_gm.num_send_tokens == 0 )
		;
	wiregl_gm.num_outstanding_writes++;
	wiregl_gm.num_send_tokens--;

#if WIREGL_GM_DEBUG_XXX
	wireGLWarning( WIREGL_WARN_DEBUG, "GM: sending a message to node_id=%d "
				   "start=0x%p len=%d buf=0x%p", conn->gm_node_id, start,
				   len, *bufp );
#endif

    gm_send_to_peer_with_callback( wiregl_gm.port, start, 
								   wiregl_gm.message_size, len,
								   WIREGL_GM_PRIORITY, conn->gm_node_id,
								   wiregl_gm_send_callback, *bufp );

	*bufp = NULL;
}

static void
wiregl_gm_flow_control_callback( struct gm_port *port, void *ctx, gm_status_t status )
{
	void *buf = ctx;

    if ( status != GM_SUCCESS )
    {
        wireGLSimpleError( "wiregl_gm_flow_control_callback error: %s (%d)",
                           wiregl_gm_str_error( status ), status );
    }

#if WIREGL_GM_DEBUG
	wireGLWarning( WIREGL_WARN_DEBUG, "GM: received a flow control writeback,"
				   "buf=0x%p", buf );
#endif

	wireGLBufferPoolPush( &wiregl_gm.flow_control_pool, buf );

	wireGLAssert( wiregl_gm.num_outstanding_flow > 0 );
	wiregl_gm.num_outstanding_flow--;
	wiregl_gm.num_send_tokens++;
	((void) port);
}

UTIL_DECL void
wireGLGmFlowControl( WireGLConnection *conn, int enable )
{
	void *buf;
	unsigned int *data;

	wireGLAssert( wiregl_gm.initialized );

	/* This could be trouble, because wireGLGmRecv() can in turn cause
       the receive handler to be called, which in turn could cause the
       flow control handler to be called again, *before* this call has
       completed, etc.  Bleah.  Keep an eye open. */
	while ( wiregl_gm.num_send_tokens == 0 )
		wireGLGmRecv( );
	wiregl_gm.num_outstanding_flow++;
	wiregl_gm.num_send_tokens--;

	buf = wireGLBufferPoolPop( &wiregl_gm.flow_control_pool );
	if ( !buf )
	{
		wireGLError( "wireGLGmFlowControl: pool is empty?!" );
	}

	data = (unsigned int *) buf;

	data[0] = WIREGL_GM_FLOW_CONTROL_MAGIC;
	data[1] = ( enable != 0 );

#if WIREGL_GM_DEBUG
	wireGLWarning( WIREGL_WARN_DEBUG, "GM: sending a flow control message to "
				   "node_id=%d, data=%d, buf=0x%p", 
				   conn->gm_node_id, enable, buf );
#endif

	gm_send_to_peer_with_callback( wiregl_gm.port, buf, 
								   WIREGL_GM_FLOW_CONTROL_SIZE,
								   WIREGL_GM_FLOW_CONTROL_LENGTH,
								   WIREGL_GM_PRIORITY, conn->gm_node_id,
								   wiregl_gm_flow_control_callback, buf );
}

static int 
wiregl_gm_looks_like_same_host( const char *a, const char *b )
{
	wireGLAssert( a && b );
	while ( *a && *b ) {
		if ( tolower(*a) != tolower(*b) )
			return 0;
		a++;
		b++;
	}

	/* they're the same if the names match, or if one is a DNS prefix
       of the other */
	return ( ( *a == '\0' && *b == '\0' ) ||
			 ( *a == '.'  && *b == '\0' ) ||
			 ( *a == '\0' && *b == '.'  ) );
}

UTIL_DECL unsigned int 
wireGLGmInit( WireGLNetReceiveFunc recvFunc, WireGLNetCloseFunc closeFunc )
{
	gm_status_t status;
	unsigned int node_id, max_node_id, i;
	unsigned int stride, count, n_bytes;
	void *mem;

    if ( wiregl_gm.initialized )
	{
		wireGLAssert( wiregl_gm.recv == recvFunc );
		wireGLAssert( wiregl_gm.close == closeFunc );
		return wiregl_gm.node_id;
	}

	wireGLWarning( WIREGL_WARN_DEBUG, "GM: initializing" );

	status = gm_init( );
	if ( status != GM_SUCCESS )
	{
		wireGLSimpleError( "GM: gm_init() failed: %s (%d)",
						   wiregl_gm_str_error( status ), status );
	}

	status = gm_open( &wiregl_gm.port, 0, WIREGL_GM_PORT,
					  "WireGL GM", WIREGL_GM_API_VERSION );
	if ( status != GM_SUCCESS )
	{
		wireGLSimpleError( "GM: gm_open( port=%u ) failed: %s (%d)",
						   WIREGL_GM_PORT, wiregl_gm_str_error( status ),
						   status );
	}

	wiregl_gm.message_size = gm_min_size_for_length( __wiregl_max_send );
	if ( wiregl_gm.message_size == WIREGL_GM_FLOW_CONTROL_SIZE )
		wiregl_gm.message_size++;

	wiregl_gm_info( );

	gm_max_node_id_inuse( wiregl_gm.port, &max_node_id );

	wiregl_gm.num_nodes = max_node_id + 1;
	wiregl_gm.enable = (unsigned char *) 
		wireGLAlloc( wiregl_gm.num_nodes * sizeof(wiregl_gm.enable[0]) );
	for ( i = 0; i < wiregl_gm.num_nodes; i++ )
		wiregl_gm.enable[i] = 1;

	wiregl_gm.multi = (WireGLMultiBuffer *)
		wireGLAlloc( wiregl_gm.num_nodes * sizeof(wiregl_gm.multi[0]) );
	for ( i = 0; i < wiregl_gm.num_nodes; i++ )
	{
		wiregl_gm.multi[i].len = 0;
		wiregl_gm.multi[i].max = 0;
		wiregl_gm.multi[i].buf = NULL;
	}

    gm_get_node_id( wiregl_gm.port, &node_id );
	wiregl_gm.node_id = node_id;

	wireGLBufferPoolInit( &wiregl_gm.read_pool, 16 );
	stride  = __wiregl_max_send;
	count   = WIREGL_GM_PINNED_READ_MEMORY_SIZE / stride;
	n_bytes = count * stride;
	wireGLWarning( WIREGL_WARN_DEBUG, "GM: read pool: %u buffers, %u bytes",
				   count, n_bytes );
	mem     = wiregl_gm_dma_malloc( n_bytes );
	wireGLBufferPoolLoad( &wiregl_gm.read_pool, mem, stride, count );
	for ( i = 0; i < 64; i++ )
	{
		void *buf = wireGLBufferPoolPop( &wiregl_gm.read_pool );
		if ( buf )
		{
			*(unsigned int *) buf = 0xdeadbeef;
			gm_provide_receive_buffer( wiregl_gm.port, buf,
									   wiregl_gm.message_size,
									   WIREGL_GM_PRIORITY );
		} 
	}

	wireGLBufferPoolInit( &wiregl_gm.unpinned_read_pool, 16 );

	wireGLBufferPoolInit( &wiregl_gm.write_pool, 16 );
	stride  = __wiregl_max_send;
	count   = WIREGL_GM_PINNED_WRITE_MEMORY_SIZE / stride;
	n_bytes = count * stride;
	wireGLWarning( WIREGL_WARN_DEBUG, "GM: write pool: %u buffers, %u bytes",
				   count, n_bytes );
	mem     = wiregl_gm_dma_malloc( n_bytes );
	wireGLBufferPoolLoad( &wiregl_gm.write_pool, mem, stride, count );

	wireGLBufferPoolInit( &wiregl_gm.flow_control_pool, 16 );
	stride  = WIREGL_GM_FLOW_CONTROL_LENGTH;
	count   = 64;
	n_bytes = count * stride;
	wireGLWarning( WIREGL_WARN_DEBUG, "GM: flow pool: %u buffers, %u bytes",
				   count, n_bytes );
	mem     = wiregl_gm_dma_malloc( n_bytes );
	wireGLBufferPoolLoad( &wiregl_gm.flow_control_pool, mem, stride, count );
	for ( i = 0; i < 8; i++ )
	{
		void *buf = wireGLBufferPoolPop( &wiregl_gm.flow_control_pool );
		wireGLAssert( buf );
		*(unsigned int *) buf = 0xdeadbeef;
		gm_provide_receive_buffer( wiregl_gm.port, buf,
								   WIREGL_GM_FLOW_CONTROL_SIZE,
								   WIREGL_GM_PRIORITY );
	}

	wiregl_gm.recv  = recvFunc;
	wiregl_gm.close = closeFunc;

	wiregl_gm.num_outstanding_writes = 0;
	wiregl_gm.num_outstanding_flow = 0;
	wiregl_gm.num_send_tokens = gm_num_send_tokens( wiregl_gm.port );

	wiregl_gm.initialized = 1;

	return wiregl_gm.node_id;
}

UTIL_DECL void 
wireGLGmConnection( WireGLConnection *conn )
{
	char *actual_name;

	wireGLAssert( wiregl_gm.initialized );

    if ( conn->gm_node_id == GM_NO_SUCH_NODE_ID )
    {
        wireGLSimpleError( "GM: there's no host called \"%s\"?",
						   conn->hostname );
    }

	actual_name = gm_node_id_to_host_name( wiregl_gm.port, conn->gm_node_id );
	if ( !actual_name )
	{
		wireGLSimpleError( "GM: gm_node_id_to_host_name( id=%u ) failed",
						   conn->gm_node_id );
	}

	if ( conn->hostname && 
		 !wiregl_gm_looks_like_same_host( conn->hostname, actual_name ) )
	{
		wireGLSimpleError( "GM: \"%s\" says id=%u, but I think \"%s\" "
						   "is on that ID", conn->hostname,
						   conn->gm_node_id, actual_name );
	}

#if 0
    /* don't close the socket, it shows we are alive! */
    wireGLCloseSocket( conn->tcp_socket );
#endif

    conn->type        = WIREGL_GM;
	conn->Alloc  	  = wireGLGmAlloc;
	conn->Send   	  = wireGLGmSend;
	conn->FlowControl = wireGLGmFlowControl;

    wireGLWarning( WIREGL_WARN_DEBUG, "GM: host=%s id=%d (gm_name=%s)",
                   conn->hostname, conn->gm_node_id, actual_name );
}
