/*
** util/net.c
** Generated by opengl_stub/windows_exports.py
** http://graphics.stanford.edu/software/wiregl
**
** Copyright 2000 
** The Board of Trustees of The Leland Stanford Junior University.
** All rights reserved.
**
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or
** modify this software, provided that the above copyright notice and
** this permission notice appear in all copies of this software.  No
** part of this software or any derivatives thereof may be used in
** graphics systems for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind,
** express, implied or otherwise, including without limitation, any
** warranty of merchantability or fitness for a particular purpose.
**
*/

#include "__utilsource.h"

#ifdef WINDOWS
#define WIN32_LEAN_AND_MEAN
#pragma warning( push, 3 )
#include <winsock2.h>
#pragma warning( pop )
#pragma warning( disable : 4514 )
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <unistd.h>
#endif

#include <stdio.h>
#include <stdlib.h>

#include "wiregl/include/wiregl_util.h"
#include "wiregl/include/wiregl_common.h"
#include "wiregl/include/wiregl_protocol.h"

unsigned int __wiregl_max_send = 0;

static struct {
	int                  initialized;
	WireGLNetReceiveFunc recv;
	WireGLNetCloseFunc   close;
	int                  use_gm;
} wiregl_net;

UTIL_DECL void 
wireGLPrintByteArray( void *buf, int len, int max_print )
{
    int i;

    fputs( "(", stdout );
    for ( i = 0; i < len && i < max_print; i++ )
    {
        printf( ( i ? " %02x" : "%02x" ), ((unsigned char *) buf)[i] );
    }
    if ( i < len )
        fputs( " ...", stdout );
    fputs( ")\n", stdout );
}

static void
wireGLFrobSocket( WireGLSocket sock )
{
    int sndbuf = 64*1024;
	int rcvbuf = sndbuf;
	int tcp_nodelay = 1;
#if 0
	int x, len;
#endif

    if ( setsockopt( sock, SOL_SOCKET, SO_SNDBUF,
					 (char *) &sndbuf, sizeof(sndbuf) ) )
	{
		int err = wireGLTcpipErrno( );
		wireGLWarning( WIREGL_WARN_CRITICAL, "setsockopt( SO_SNDBUF=%d ) : %s",
					   sndbuf, wireGLTcpipErrorString( err ) );
	}

    if ( setsockopt( sock, SOL_SOCKET, SO_RCVBUF,
					 (char *) &rcvbuf, sizeof(rcvbuf) ) )
	{
		int err = wireGLTcpipErrno( );
		wireGLWarning( WIREGL_WARN_CRITICAL, "setsockopt( SO_RCVBUF=%d ) : %s",
					   rcvbuf, wireGLTcpipErrorString( err ) );
	}

	if ( setsockopt( sock, IPPROTO_TCP, TCP_NODELAY,
					 (char *) &tcp_nodelay, sizeof(tcp_nodelay) ) )
	{
		int err = wireGLTcpipErrno( );
		wireGLWarning( WIREGL_WARN_CRITICAL, "setsockopt( TCP_NODELAY=%d )"
					   " : %s", tcp_nodelay, wireGLTcpipErrorString( err ) );
	}

#if 0
	x = 0xdeadbeef;
	len = sizeof(x);
	getsockopt( sock, SOL_SOCKET, SO_SNDBUF,
				(char *) &x, &len );
	printf( "sndbuf=%d (asked for %d)\n", x, sndbuf );

	x = 0xdeadbeef;
	len = sizeof(x);
	getsockopt( sock, SOL_SOCKET, SO_RCVBUF,
				(char *) &x, &len );
	printf( "rcvbuf=%d (asked for %d)\n", x, rcvbuf );

	x = 0xdeadbeef;
	len = sizeof(x);
	getsockopt( sock, IPPROTO_TCP, TCP_NODELAY,
				(char *) &x, &len );
	printf( "tcp_nodelay=%d (asked for %d)\n", x, tcp_nodelay );
#endif
}

UTIL_DECL WireGLConnection *
wireGLConnectToServer( char *server, unsigned short default_port, WireGLConnectionRequest *request )
{
    struct sockaddr_in servaddr;
    struct hostent *hp;
    char hostname[4096], protocol[4096];
    unsigned short port;
	unsigned int request_size;
    WireGLConnection *conn;
    WireGLConnectionRequest local_request;
	WireGLConnectionRequest *p;

	wireGLAssert( wiregl_net.initialized );
	wireGLTcpipInit( wiregl_net.recv, wiregl_net.close );

    wireGLWarning( WIREGL_WARN_DEBUG, "Connecting to %s", server );

    if (!request)
    {
        request = &local_request;
    }
    else
    {
        // windows doesn't let us directly set this global variable
        // (cross DLL mumble-mumble), so sneak in the value
        wireGLAssert( __wiregl_max_send == 0 ||
                      __wiregl_max_send == request->max_send );
        __wiregl_max_send = request->max_send;
    }

    request->magic    = WIREGL_CONNECTION_MAGIC;
    request->max_send = __wiregl_max_send;

    if (!wireGLParseURL( server, protocol, hostname, &port, default_port ))
    {
        wireGLSimpleError( "Malformed URL: \"%s\"", server );
    }

    conn = (WireGLConnection *) wireGLAlloc( sizeof( *conn ) );

	conn->type        = WIREGL_NO_CONNECTION;
	conn->sender_id   = 0;
    conn->hostname    = strdup( hostname );
	conn->Alloc       = NULL;
	conn->Send        = NULL;
	conn->FlowControl = NULL;
	conn->tcp_socket  = 0;
	conn->gm_node_id  = 0;

    request->please_share_memory = 0;
    request->please_use_gm = 0;
	request->gm_client_id = 0; /* GM_NO_SUCH_NODE_ID */

    if ( !strcmp( protocol, "devnull" ) )
    {
#if 1
		wireGLDevNullInit( wiregl_net.recv, wiregl_net.close );
		wireGLDevNullConnection( conn );
#else
		wireGLSimpleError( "drop packet not supported right now" );
#endif
        return conn;
    }
    else if ( !strcmp( protocol, "file" ) )
    {
#if 0
		wireGLFileConnection( conn, "w" );
#else
		wireGLSimpleError( "file protocol is broken" );
#endif
        return conn;
    }
#ifdef GM_SUPPORT
    else if ( !strcmp( protocol, "gm" ) )
    {
		/* just note that we are trying to setup GM, we'll have to
           finish this later... */
        conn->type = WIREGL_GM;
        request->please_use_gm = 1;
		request->gm_client_id = 
			wireGLGmInit( wiregl_net.recv, wiregl_net.close );
    }
#endif
    else if ( !strcmp( protocol, "tcpip" ) )
    {
		conn->type = WIREGL_TCPIP;
	}
	else
	{
        wireGLSimpleError( "Unknown Protocol: \"%s\"", protocol );
    }

    conn->tcp_socket = socket( AF_INET, SOCK_STREAM, 0 );
    if ( conn->tcp_socket < 0 )
	{
		int err = wireGLTcpipErrno( );
        wireGLSimpleError( "socket error: %s", wireGLTcpipErrorString( err ) );
	}

	wireGLFrobSocket( conn->tcp_socket );

    memset( &servaddr, 0, sizeof( servaddr ) );
    servaddr.sin_family = AF_INET;
    servaddr.sin_port = htons( port );

    hp = gethostbyname( hostname );
    if ( !hp )
    {
        wireGLSimpleError( "Unknown host: \"%s\"", hostname );
    }
    memcpy( (char *) &servaddr.sin_addr, hp->h_addr,
            sizeof( servaddr.sin_addr ) );
    if ( connect( conn->tcp_socket, (struct sockaddr *) &servaddr,
				  sizeof( servaddr )) < 0 )
    {
		int err = wireGLTcpipErrno( );
        wireGLSimpleError( "Couldn't connect to %s:%d, %s",
                           hostname, port, wireGLTcpipErrorString( err ) );
    }

    if ( request->max_send < 1024 )
    {
        wireGLSimpleError( "wireGLConnectToServer: max_send too small (%d)\n",
                           request->max_send );
    }


	/* calculate the request size */
	request_size = sizeof(*request);
	for (p = request; *(p->peer); p++)
		request_size += sizeof(*request);

	wireGLTcpipWriteExact( conn->tcp_socket, request, request_size );

    return conn;
}

UTIL_DECL void 
wireGLFinishConnectToServer( WireGLConnection *conn )
{
    WireGLConnectionResponse response;

    /* IB: Temporary fix for devnull */
    if ( conn->type == WIREGL_DROP_PACKETS )
    {
        static int nullid = 0;
        conn->sender_id = nullid++;
        return;
    }

	wireGLTcpipReadExact( conn->tcp_socket, &response, sizeof(response) );

    if ( response.magic != WIREGL_CONNECTION_MAGIC )
    {
        wireGLSimpleError( "wireGLFinishConnectToServer: connection "
                           "magic=0x%x, expected 0x%x",
                           response.magic, WIREGL_CONNECTION_MAGIC );
    }

    if ( response.max_send < __wiregl_max_send )
    {
        wireGLSimpleError( "wireGLFinishConnectToServer: server has a "
                           "max_send=%d, but client is using %d",
                           response.max_send, __wiregl_max_send );
    }

    conn->sender_id = response.client_id;

	switch ( conn->type )
	{
#ifdef GM_SUPPORT
	  case WIREGL_GM:
		/* set this up as a TCPIP connection first, so that we can
         * detect a client disconnect via socket death */
		wireGLTcpipConnection( conn );

		/* and now set it up correctly for GM */
		conn->gm_node_id = response.gm_server_id;
		wiregl_net.use_gm++;
        wireGLGmConnection( conn );
		break;
#endif
	  case WIREGL_TCPIP:
		wireGLTcpipConnection( conn );
		break;

	  default:
		wireGLError( "wireGLFinishConnectToServer: conn->type=%u?",
					 conn->type );
	}

    wireGLWarning( WIREGL_WARN_DEBUG, "Done connecting to server." );
}

#if defined( WINDOWS ) || defined( IRIX ) || defined( IRIX64 )
typedef int socklen_t;
#endif

UTIL_DECL WireGLServer *
wireGLBecomeServer( unsigned short port )
{
    WireGLServer *server = (WireGLServer *) wireGLAlloc( sizeof( *server ) );

	wireGLAssert( wiregl_net.initialized );
	wireGLTcpipInit( wiregl_net.recv, wiregl_net.close );

	server->sock = socket( AF_INET, SOCK_STREAM, 0 );
    if (server->sock == -1)
    {
        wireGLSimpleError( "Couldn't create socket" );
    }

	wireGLFrobSocket( server->sock );

    server->addr.sin_family = AF_INET;
    server->addr.sin_addr.s_addr = INADDR_ANY;
    server->addr.sin_port = htons( port );
    if ( bind( server->sock, (struct sockaddr *) &server->addr,
               sizeof( server->addr ) ) == -1 )
    {
		int err = wireGLTcpipErrno( );
        wireGLSimpleError( "Couldn't bind to socket (port=%d): %s",
						   port, wireGLTcpipErrorString( err ) );
    }

    return server;
}

UTIL_DECL WireGLConnection *
wireGLAcceptClient( WireGLServer *server, WireGLConnectionRequest **request )
{
    WireGLConnection *conn = 
		(WireGLConnection *) wireGLAlloc( sizeof( *conn ) );
	int i;

    socklen_t addr_length = sizeof( server->addr );

    listen( server->sock, 8 /* max pending connections */ );

    conn->tcp_socket = accept( server->sock,
                               (struct sockaddr *) &server->addr,
                               &addr_length );
    if (conn->tcp_socket == -1)
    {
        wireGLSimpleError( "Accept error" );
    }

    conn->type        = WIREGL_TCPIP;
	conn->sender_id   = 0;
	conn->hostname    = NULL;

	conn->Alloc       = NULL;
	conn->Send        = NULL;
	conn->FlowControl = NULL;

	conn->gm_node_id  = 0; /* GM_NO_SUCH_NODE_ID */

	
	*request = 
		(WireGLConnectionRequest*) wireGLAlloc (sizeof (**request));
	for (i=0; ; i++) {
		wireGLTcpipReadExact( conn->tcp_socket, (*request)+i, sizeof(**request) );
		if (!*((*request)+i)->peer) break;
		/* grow requests */
		wireGLRealloc ((void **) request, sizeof (**request) * (i+2));
	}

    if ( (*request)->magic != WIREGL_CONNECTION_MAGIC )
    {
        wireGLSimpleError( "wireGLAcceptClient: connection "
                           "magic=0x%x, expected 0x%x\n",
                           (*request)->magic, WIREGL_CONNECTION_MAGIC );
    }
    if ( __wiregl_max_send == 0 )
    {
        __wiregl_max_send = (*request)->max_send;
        wireGLWarning( WIREGL_WARN_NOTICE, "pipeserver: setting max_send=%d",
                       __wiregl_max_send );
    }
    else
    {
        if ( (*request)->max_send > __wiregl_max_send )
        {
            wireGLSimpleError( "wireGLAcceptClient: client specified "
                               "max_send=%d, but server already "
                               "using %d\n", (*request)->max_send,
                               __wiregl_max_send );
        }
    }

    if ( (*request)->please_use_gm )
    {
#ifdef GM_SUPPORT
        conn->type = WIREGL_GM;
		conn->gm_node_id = (*request)->gm_client_id;
#else
        wireGLSimpleError( "wireGLAcceptClient: client has "
                           "requested GM, but we don't support it." );
#endif
    }

    return conn;
}

UTIL_DECL void 
wireGLFinishAcceptClient( WireGLConnection *conn, WireGLConnectionRequest *request )
{
    static int num_clients = 0;
    WireGLConnectionResponse response;

	wireGLFree (request);

    response.magic     = WIREGL_CONNECTION_MAGIC;
    response.max_send  = __wiregl_max_send;
    response.client_id = num_clients++;
    response.gm_server_id = 0; /* GM_NO_SUCH_NODE_ID */
#ifdef GM_SUPPORT
    if ( conn->type == WIREGL_GM )
    {
		response.gm_server_id = 
			wireGLGmInit( wiregl_net.recv, wiregl_net.close );
    }
#endif

    conn->sender_id = response.client_id;

	wireGLTcpipWriteExact( conn->tcp_socket, &response, sizeof(response) );

    switch( conn->type )
    {
#ifdef GM_SUPPORT
	  case WIREGL_GM:
		wireGLTcpipConnection( conn );
		wireGLGmConnection( conn );
		wiregl_net.use_gm++;
		break;
#endif
	  case WIREGL_TCPIP:
		wireGLTcpipConnection( conn );
		break;

	  default:
		wireGLSimpleError( "wireGLFinishAcceptClient: "
						   "unknown conn->type=%u", conn->type );
		break;
    }
}

UTIL_DECL void
wireGLNetInit( WireGLNetReceiveFunc recvFunc, WireGLNetCloseFunc closeFunc )
{
	if ( wiregl_net.initialized )
		return;

	wiregl_net.recv   = recvFunc;
	wiregl_net.close  = closeFunc;
	wiregl_net.use_gm = 0;

	wiregl_net.initialized = 1;
}

UTIL_DECL void *
wireGLNetAlloc( WireGLConnection *conn )
{
	wireGLAssert( conn );
	return conn->Alloc( );
}

UTIL_DECL void
wireGLNetSend( WireGLConnection *conn, void **bufp, 
			   void *start, unsigned int len )
{
	wireGLAssert( conn );
	wireGLAssert( len > 0 );
	wireGLAssert( bufp != NULL );
	wireGLAssert( start >= *bufp );
	wireGLAssert( (unsigned char *) start + len <= 
				  (unsigned char *) *bufp + __wiregl_max_send );

	conn->Send( conn, bufp, start, len );
}

UTIL_DECL void
wireGLNetRecv( void )
{
	wireGLTcpipRecv( );
#ifdef GM_SUPPORT
	if ( wiregl_net.use_gm )
		wireGLGmRecv( );
#endif
	/* HACK */
	wireGLSleep( 0 );
}

UTIL_DECL void 
wireGLNetMultiWrite( WireGLConnection *conn, void *buf, unsigned int len )
{
	unsigned char *src;
	wireGLAssert( buf != NULL && len > 0 );

	if ( conn->type == WIREGL_TCPIP )
	{
		wireGLTcpipWriteExact( conn->tcp_socket, &len, sizeof(len) );
		wireGLTcpipWriteExact( conn->tcp_socket, buf, len );
		return;
	}

	src = (unsigned char *) buf;
	while ( len > 0 )
	{
		void               *buf = conn->Alloc( );
		WireGLMessageMulti *msg = (WireGLMessageMulti *) buf;
		unsigned char      *dst = (unsigned char *) ( msg + 1 );
		unsigned int        n_bytes;

		if ( len + sizeof(*msg) > __wiregl_max_send )
		{
			msg->type     = WIREGL_MESSAGE_MULTI_BODY;
			msg->senderId = conn->sender_id;
			n_bytes       = __wiregl_max_send - sizeof(*msg);
		}
		else
		{
			msg->type     = WIREGL_MESSAGE_MULTI_TAIL;
			msg->senderId = conn->sender_id;
			n_bytes       = len;
		}
		memcpy( dst, src, n_bytes );

		conn->Send( conn, &buf, buf, n_bytes + sizeof(*msg) );

		src += n_bytes;
		len -= n_bytes;
	}
}

UTIL_DECL unsigned int
wireGLNetMaxSend( void )
{
	return __wiregl_max_send;
}
