/*
** util/tcpip.c
** Generated by opengl_stub/windows_exports.py
** http://graphics.stanford.edu/software/wiregl
**
** Copyright 2000 
** The Board of Trustees of The Leland Stanford Junior University.
** All rights reserved.
**
** Except for commercial resale, lease, license or other commercial
** transactions, permission is hereby given to use, copy, and/or
** modify this software, provided that the above copyright notice and
** this permission notice appear in all copies of this software.  No
** part of this software or any derivatives thereof may be used in
** graphics systems for resale or for use in a commercial product.
**
** This software is provided "as is" and without warranty of any kind,
** express, implied or otherwise, including without limitation, any
** warranty of merchantability or fitness for a particular purpose.
**
*/

#include "__utilsource.h"

#ifdef WINDOWS
#define WIN32_LEAN_AND_MEAN
#pragma warning( push, 3 )
#include <winsock2.h>
#pragma warning( pop )
#pragma warning( disable : 4514 )
#pragma warning( disable : 4127 )
typedef int ssize_t;
#define write(a,b,c) send(a,b,c,0)
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#endif

#include <limits.h>
#include <stdio.h>
#include <errno.h>

#include "wiregl/include/wiregl_util.h"

#ifdef WINDOWS

#undef  ECONNRESET
#define ECONNRESET  WSAECONNRESET
#undef  EINTR
#define EINTR       WSAEINTR

UTIL_DECL int wireGLTcpipErrno( void )
{
	return WSAGetLastError( );
}

UTIL_DECL char *wireGLTcpipErrorString( int err )
{
    static char buf[512], *temp;

	sprintf( buf, "err=%d", err );

#define X(x)	strcpy(buf,x); break

	switch ( err )
	{
	  case WSAECONNREFUSED: X( "connection refused" );
	  case WSAECONNRESET:   X( "connection reset" );
	  default:
		FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER |
					   FORMAT_MESSAGE_FROM_SYSTEM |
					   FORMAT_MESSAGE_MAX_WIDTH_MASK, NULL, err,
					   MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ),
					   (LPTSTR) &temp, 0, NULL );
		if ( temp )
		{
			strncpy( buf, temp, sizeof(buf)-1 );
			buf[ sizeof(buf)-1 ] = '\0';
		}
	}

#undef X

	temp = buf + strlen(buf) - 1;
	while ( temp > buf && isspace( *temp ) )
	{
		*temp = '\0';
		temp--;
	}

	return buf;
}

#else

UTIL_DECL int wireGLTcpipErrno( void )
{
	return errno;
}

UTIL_DECL char *wireGLTcpipErrorString( int err )
{
	static char buf[512], *temp;
	
	temp = strerror( err );
	if ( temp )
	{
		strncpy( buf, temp, sizeof(buf)-1 );
		buf[ sizeof(buf)-1 ] = '\0';
	}
	else
	{
		sprintf( buf, "err=%d", err );
	}

	return buf;
}

#endif

UTIL_DECL void 
wireGLCloseSocket( WireGLSocket sock )
{
#ifdef WINDOWS
    closesocket( sock );
#else
    close( sock );
#endif
}

#if 0
/**************************************************************************/

static FILE *log_file = NULL;

static void
wireGLLogOpen( void )
{
	char *file;
	log_file = NULL;
	file = getenv( "WIREGL_LOG" );
	if ( file )
	{
		wireGLWarning( WIREGL_WARN_DEBUG, "wireGLLogOpen: logging to \"%s\"",
					   file );
		log_file = fopen( file, "wb" );
		if ( log_file )
		{
			unsigned __int64 freq;
			QueryPerformanceFrequency( (LARGE_INTEGER *) &freq );
			fprintf( log_file, "freq %I64u\n", freq );
		}
		else
		{
			wireGLWarning( WIREGL_WARN_CRITICAL, "wireGLLogOpen: trouble "
						   "opening \"%s\"", file );
		}
	}
}

static void
wireGLLogRead( unsigned int len )
{
	if ( log_file ) 
	{
		unsigned __int64 time;
		QueryPerformanceCounter( (LARGE_INTEGER *) &time );
		fprintf( log_file, "read %u %I64u\n", len, time );
	}
}

static void
wireGLLogWrite( unsigned int len )
{
	if ( log_file ) 
	{
		unsigned __int64 time;
		QueryPerformanceCounter( (LARGE_INTEGER *) &time );
		fprintf( log_file, "write %u %I64u\n", len, time );
	}
}

static void
wireGLLogClose( void )
{
	if ( log_file )
	{
		fclose( log_file );
		log_file = NULL;
	}
}

#endif

/**************************************************************************/

typedef struct {
    unsigned int  num;
    unsigned int  max;
    WireGLSocket *socket;
} WireGLSocketSet;

static void
wireGLSocketSetInit( WireGLSocketSet *set )
{
    set->num = 0;
    set->max = 8;
    set->socket = 
		(WireGLSocket *) wireGLAlloc( set->max * sizeof(set->socket[0]) );
}

static void 
wireGLSocketSetAdd( WireGLSocketSet *set, WireGLSocket sock )
{
    if ( set->num == set->max )
    {
		int n_bytes;
        set->max <<= 1;
		n_bytes = set->max * sizeof(set->socket[0]);
		wireGLRealloc( (void **) &set->socket, n_bytes );
    }
    set->socket[set->num++] = sock;
}

static struct {
	int                  initialized;
	fd_set               read_fds;
	int                  max_fd;
	WireGLSocketSet      conns;
	WireGLBufferPool     bufpool;
	WireGLSocketSet      incoming;
	WireGLLock           lock;
	WireGLNetReceiveFunc recv;
	WireGLNetCloseFunc   close;
} wiregl_tcpip;

static int TcpipReadExact( WireGLSocket sock, void *buf, unsigned int len )
{
    char *dst = (char *) buf;

    while ( len > 0 )
    {
        int num_read = recv( sock, dst, (int) len, 0 );

#ifdef WINDOWS_XXXX
		/* MWE: why is this necessary for windows???  Does it return a
           "good" value for num_bytes despite having a reset
           connection? */
		if ( wireGLTcpipErrno( ) == ECONNRESET )
            return -1;
#endif

        if ( num_read < 0 )
		{
			if ( wireGLTcpipErrno( ) == EINTR )
			{
				wireGLWarning( WIREGL_WARN_DEBUG, "Tcpip_ReadExact: "
							   "caught an EINTR, looping for more data" );
				continue;
			}
            return -1;
		}
        
		if ( num_read == 0 ) 
		{
			/* client exited gracefully */
			return 0;
        }

		dst += num_read;
		len -= num_read;
    }

    return 1;
}

UTIL_DECL void 
wireGLTcpipReadExact( WireGLSocket sock, void *buf, unsigned int len )
{
	int retval = TcpipReadExact( sock, buf, len );
	if ( retval <= 0 )
	{
		int err = wireGLTcpipErrno( );
		wireGLSimpleError( "wireGLTcpipReadExact: %s", 
						   wireGLTcpipErrorString( err ) );
	}
}

static int TcpipWriteExact( WireGLSocket sock, void *buf, unsigned int len )
{
    char *src = (char *) buf;

    while ( len > 0 )
    {
		int num_written = write( sock, src, len );
        if ( num_written <= 0 )
        {
			if ( wireGLTcpipErrno( ) == EINTR )
			{
				wireGLWarning( WIREGL_WARN_DEBUG, "TcpipWriteExact: "
							   "caught an EINTR, continuing" );
				continue;
			}
			return -1;
        }

        len -= num_written;
        src += num_written;
    }

    return 1;
}

UTIL_DECL void 
wireGLTcpipWriteExact( WireGLSocket sock, void *buf, unsigned int len )
{
	int retval = TcpipWriteExact( sock, buf, len );
	if ( retval <= 0 )
	{
		int err = wireGLTcpipErrno( );
		wireGLSimpleError( "wireGLTcpipWriteExact: %s", 
						   wireGLTcpipErrorString( err ) );
	}
}

UTIL_DECL void *
wireGLTcpipAlloc( void )
{
	void *buf = wireGLBufferPoolPop( &wiregl_tcpip.bufpool );
	if ( buf == NULL )
	{
		buf = wireGLAlloc( __wiregl_max_send + sizeof(int) );
		/* skip over the space for the length field */
		buf = (int *) buf + 1;
	}
	return buf;
}

static void
wiregl_tcpip_free( void *buf )
{
	wireGLBufferPoolPush( &wiregl_tcpip.bufpool, buf );
}

UTIL_DECL void 
wireGLTcpipSend( WireGLConnection *conn, void **bufp,
				 void *start, unsigned int len )
{
	/* All of the buffers passed to the send function were allocated
       with wireGLTcpipAlloc(), which includes a hidden 4 bytes before
       the packet, to insure that we always have a place to write the
       length field, even when start == *bufp. */
	unsigned int *lenp = (unsigned int *) start - 1;
	*lenp = len;

	if ( TcpipWriteExact( conn->tcp_socket, lenp, len + sizeof(int) ) < 0 )
	{
		int err = wireGLTcpipErrno( );
		wireGLSimpleError( "wireGLTcpipSend: %s",
						   wireGLTcpipErrorString( err ) );
	}

#if 0
	wireGLLogWrite( len );
#endif

	/* reclaim this pointer for reuse and try to keep the client from
	   accidentally reusing it directly */
	wireGLBufferPoolPush( &wiregl_tcpip.bufpool, *bufp );
	*bufp = NULL;
}

static void 
TcpipDeadConnection( WireGLSocket sock )
{
#if 1
	int err = wireGLTcpipErrno( );
	wireGLSimpleError( "DeadConnection (sock=%d)!  I'm out of here! (%s)",
					   sock, wireGLTcpipErrorString( err ) );
#else
    WireGLConnection *conn = wireGLLookupClientConnBySocket( sock );
    handlers->disconnect( conn->sender_id );
    wireGLCloseSocket( sock );
    wireGLDeleteClientConn( conn );
    wireGLFree( conn );
#endif
}

UTIL_DECL void
wireGLTcpipRecv( void )
{
	int    num_ready;
	fd_set read_fds;
	unsigned int i;
	struct timeval timeout;
	WireGLFreeFunc freeFunc;

	if ( wiregl_tcpip.incoming.num )
	{
		wireGLAcquireLock( &wiregl_tcpip.lock );
		for ( i = 0; i < wiregl_tcpip.incoming.num; i++ )
		{
			WireGLSocket sock = wiregl_tcpip.incoming.socket[i];
			if ( (int) sock + 1 > wiregl_tcpip.max_fd )
				wiregl_tcpip.max_fd = (int) sock + 1;
			FD_SET( sock, &wiregl_tcpip.read_fds );
			wireGLSocketSetAdd( &wiregl_tcpip.conns, sock );
		}
		wiregl_tcpip.incoming.num = 0;
		wireGLReleaseLock( &wiregl_tcpip.lock );
	}

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;
	read_fds = wiregl_tcpip.read_fds;
#if 1
	num_ready = select( wiregl_tcpip.max_fd, &read_fds, NULL, NULL, &timeout );
#else
	num_ready = select( wiregl_tcpip.max_fd, &read_fds, NULL, NULL, NULL );
#endif
	if ( num_ready < 0 )
	{
		int err = wireGLTcpipErrno( );
		wireGLSimpleError( "select failed: %s", 
						   wireGLTcpipErrorString( err ) );
	}

	if ( num_ready == 0 )
		return;

	for ( i = 0; i < wiregl_tcpip.conns.num; i++ ) {

		void         *buf;
		unsigned int  len;
		int           read_ret;
		WireGLSocket  sock = wiregl_tcpip.conns.socket[i];

		if ( !FD_ISSET( sock, &read_fds ) )
			continue;

		read_ret = TcpipReadExact( sock, &len, sizeof(len) );
		if ( read_ret <= 0 )
		{
			TcpipDeadConnection( sock );
			i--;
			continue;
		}

		wireGLAssert( len > 0 );

		if ( len <= __wiregl_max_send )
		{
			buf = wireGLTcpipAlloc( );
			freeFunc = wiregl_tcpip_free;
		}
		else
		{
			buf = wireGLAlloc( len );
			freeFunc = wireGLFree;
		}

		read_ret = TcpipReadExact( sock, buf, len );
		if ( read_ret <= 0 )
		{
			TcpipDeadConnection( sock );
			freeFunc( buf );
			i--;
			continue;
		}

#if 0
		wireGLLogRead( len );
#endif

		wiregl_tcpip.recv( buf, len, freeFunc );
	}
}

UTIL_DECL void
wireGLTcpipFlowControl( WireGLConnection *conn, int enable )
{
	if ( enable )
		FD_SET( conn->tcp_socket, &wiregl_tcpip.read_fds );
	else
		FD_CLR( conn->tcp_socket, &wiregl_tcpip.read_fds );
}

UTIL_DECL void
wireGLTcpipInit( WireGLNetReceiveFunc recvFunc, WireGLNetCloseFunc closeFunc )
{
	if ( wiregl_tcpip.initialized )
	{
		wireGLAssert( wiregl_tcpip.recv == recvFunc );
		wireGLAssert( wiregl_tcpip.close == closeFunc );
		return;
	}

#if 0
	wireGLLogOpen( );
#endif

	FD_ZERO( &wiregl_tcpip.read_fds );
	wiregl_tcpip.max_fd = 0;

	wireGLSocketSetInit( &wiregl_tcpip.conns );
	wireGLBufferPoolInit( &wiregl_tcpip.bufpool, 16 );
	wireGLSocketSetInit( &wiregl_tcpip.incoming );
	wireGLInitLock( &wiregl_tcpip.lock );

	wiregl_tcpip.recv = recvFunc;
	wiregl_tcpip.close = closeFunc;

	wiregl_tcpip.initialized = 1;
}

UTIL_DECL void 
wireGLTcpipConnection( WireGLConnection *conn )
{
	wireGLAssert( wiregl_tcpip.initialized );

    conn->type        = WIREGL_TCPIP;
	conn->Alloc  	  = wireGLTcpipAlloc;
	conn->Send   	  = wireGLTcpipSend;
	conn->FlowControl = wireGLTcpipFlowControl;

	/* put this guy on a list... */
	wireGLAcquireLock( &wiregl_tcpip.lock );
	wireGLSocketSetAdd( &wiregl_tcpip.incoming, conn->tcp_socket );
	wireGLReleaseLock( &wiregl_tcpip.lock );
}
